/**
 * Copyright (c) 2007, 2008 Diomidis Spinellis and Panagiotis Louridas
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Publications using results from this work or its derivatives must
 *    acknowledge the work's origin by citing the corresponding August 2008
 *    Communications of the ACM paper.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: cortest.cpp,v 1.3 2008/06/26 09:42:56 panos Exp $
 *
 */

/**
 * Reads the output of freqs.cpp and outputs correlation tests
 * for the calculated values and the actual values.
 * 
 */

#include <iostream>
#include <map>

#include <assert.h>

#include <math.h>

#include "tokenize.h"
#include "nr.h"

using namespace std;

typedef vector<long>::const_iterator CVIT;

void 
cortests(vector<long>data1, vector<long>data2)
{
	assert(data1.size() == data2.size());
	long datasz = data1.size();
	Vec_IO_DP d1(datasz);
	Vec_IO_DP d2(datasz);
	DP d, zd, probd, rs, probrs;
	long i = 0;
	
	typedef vector<long>::const_iterator CVIT;
	CVIT c1 = CVIT(data1.begin());
	CVIT c2 = CVIT(data2.begin());
	
	while (c1 != CVIT(data1.end()) && c2 != CVIT(data2.end())) {
		d1[i] = *c1++;
		d2[i] = *c2++;
		i++;
	}

	NR::spear(d1, d2, d, zd, probd, rs, probrs);

	cout << "Spearman d=" << d << " zd=" << zd << " probd=" << probd
		<< " rs=" << rs << " probrs=" << probrs << endl;

	DP r, prob, z;
	NR::pearsn(d1, d2, r, prob, z); 
	cout << "Pearson r=" << r << " prob=" << prob << " z=" << z << endl;
	
	DP a, b, siga, sigb, chi2, q;
	Vec_IO_DP sig2(datasz);
	NR::fit(d1, d2, sig2, 0, a, b, siga, sigb, chi2, q);
	cout << "Regression a=" << a << " b=" << b << endl;
}

int
main()
{
	string line;
	vector<long> expecteds;
	vector<long> actuals;
	map<long, long> dexpecteds;
	map<long, long> dactuals;

	/*
	 * Get the expected and actual number of references added
	 * to each article each month. 
	 */
	cerr << "reading input..." << endl;
	long points = 0;
	while (getline(cin, line)) {
		vector<string> fields;
		tokenize(line, fields, ":");
		assert(fields.size() == 7);

		long expected = atoi(fields[5].c_str());
		long actual = atoi(fields[6].c_str());
	       	if (expected > 0 && actual > 0) {
			expecteds.push_back(expected);
			actuals.push_back(actual);
			dexpecteds[expected]++;
			dactuals[actual]++;
		}
	}

	cerr << "finished input..." << endl;
	cerr << actuals.size() << " actuals " << expecteds.size() << " expecteds" << endl;

	cout << "Correlations of data points:" << endl;
	cortests(expecteds, actuals);
	cout << "Correlations of distributions:" << endl;
	long datasz = dexpecteds.size() < dactuals.size()
		? dexpecteds.size() : dactuals.size();
	vector<long> t1;
	vector<long> t2;
	for (long i = 0; i < datasz; i++) {
		if (dexpecteds.find(i) != dexpecteds.end()
			&& dactuals.find(i) != dactuals.end()) {
			t1.push_back(dactuals[i]);
			t2.push_back(dexpecteds[i]);
		}
	}
	cortests(t1, t2);

}
