#!/usr/bin/perl
#
# Copyright (c) 2008 Diomidis Spinellis
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Publications using results from this work or its derivatives must
#    acknowledge the work's origin by citing the corresponding August 2008
#    Communications of the ACM paper.
#
# THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: crawl.pl,v 1.4 2008/06/26 06:30:12 dds Exp $
#
# Crawl a Wikipedia article to a specific depth producing a neato graph
#

use LWP;

my $browser = LWP::UserAgent->new;

$maxdepth = 4;
$start = 'Communications of the ACM';
$limit = 5;

open(OUT, '>' . ($f = "depth$maxdepth-$limit.neato")) || die "Unable to open $f: $!\n";
print OUT qq(
/*
 * Start: $start
 * Maxdepth: $maxdepth
 * Limit: $limit
 */

graph Wikipedia {
	node [shape=plaintext,height=0.2,fontname="Helvetica",fontsize=8];
);

crawl($start, 0);

print OUT "}\n";

sub
crawl
{
	my($name, $depth) = @_;

	return if ($done{$name});
	$done{$name} = 1;

	print STDERR "Doing $name\n";
	my $url = "http://en.wikipedia.org/wiki/Special:Export/$name";
	my $response = $browser->get($url);
	die "Couldn't get $url -- ", $response->status_line unless $response->is_success;

	die "I was expecting XML, not ", $response->content_type, $response->content unless $response->content_type =~ m|application/xml|;
	my $content = $response->content;
	# print "$content\n";

	# Obtain id as a type of timestamp
	if ($content =~ m/<id>(\d+)<\/id>/) {
		print OUT qq{\t"$name"; // $1\n};
	} else {
		print OUT qq{\t"$name";\n};
	}
	return if ($depth++ == $maxdepth);
	my $count = 0;
	while ($content =~ s/\[\[(.*?)(\]\]|(\|.*?\]\]))/DONE/) {
		my $newname = $1;
		next if ($newname =~ /[#:}]/);
		last if ($count++ == $limit);
		# Change first character to uppercase
		$newname =~ s/^(.)/uc($1)/e;
		print OUT qq{\t"$name" -- "$newname";\n};
		crawl($newname, $depth);
	}
}
