/**
 * Copyright (c) 2007, 2008 Diomidis Spinellis and Panagiotis Louridas
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Publications using results from this work or its derivatives must
 *    acknowledge the work's origin by citing the corresponding August 2008
 *    Communications of the ACM paper.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: freqs.cpp,v 1.5 2008/06/26 09:42:56 panos Exp $
 *
 */

/*
 * Reads the output of wikipedia handling and outputs 
 * outputs lines of the form:
 * month:name:added:start:end:expected:actual
 *
 */

#include <iostream>
#include <string>
#include <vector>
#include <utility>

#include <assert.h>

#include "tokenize.h"

using namespace std;

typedef pair<string, vector<long> > ArticleReferences;

const int NUM_FIELDS = 82;
const int MONTHS_END = NUM_FIELDS - 1;
const int MONTHS_START = 10;
const int NUM_MONTHS = MONTHS_END - MONTHS_START;

int
main()
{
	assert(NUM_MONTHS == 71);

	string line;
	vector<long> monthSums = vector<long>(NUM_MONTHS, 0);
	vector<ArticleReferences*> refsData;
	long article = 0;

	/*
	 * Get the total number of references per month and
	 * the number of references of each article for each month.
	 */
	while (getline(cin, line))
	{
		vector<string> fields;
		tokenize(line, fields, ":");
		if (fields.size() == NUM_FIELDS)
		{
			ArticleReferences *articleReferences =
				new ArticleReferences();
			articleReferences->first = fields[0];
			for (int i = 0; i < NUM_MONTHS; i++)
			{
				long monthRefs =
					atol(fields[i + MONTHS_START].c_str());
				if (articleReferences->second.size() > 0
					&& articleReferences->second.back() > 0)
					monthSums[i] = monthSums[i] + monthRefs;
				articleReferences->second.push_back(monthRefs);
			}
			refsData.push_back(articleReferences);
			article++;
			if (article % 100000 == 0) cerr << article << endl;
		} else {
			cerr << "fields: " << fields.size() << ":" << line << endl;
		}
	}
	cerr << article << endl;

	cout << "# month:name:added:start:end:expected:actual" << endl;
	/*
	 * Calculate the expected number of references added to each
	 * article per month and the actual number of references it got.
	 */
	for (vector<ArticleReferences*>::iterator it = refsData.begin();
	     it != refsData.end(); it++)
	{
		for (int i = 0; i < NUM_MONTHS - 1; i++)
		{
			if (monthSums[i] == 0 || (*it)->second[i] == 0) 
				continue;
			long expected = (monthSums[i+1] - monthSums[i])
				* (*it)->second[i] / monthSums[i];
			long actual = (*it)->second[i+1] - (*it)->second[i];
			if (expected != 0 || actual != 0)
			{
				cout << i
				     << ":"
				     << (*it)->first
				     << ":"
				     << (monthSums[i+1] - monthSums[i])
				     << ":"
				     << (*it)->second[i]
				     << ":"
				     << (*it)->second[i+1]
				     << ":" << expected
				     << ":"
				     << actual
				     << endl;
			}
		}
	}
}

  
