/**
 * Copyright (c) 2007, 2008 Diomidis Spinellis and Panagiotis Louridas
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Publications using results from this work or its derivatives must
 *    acknowledge the work's origin by citing the corresponding August 2008
 *    Communications of the ACM paper.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: prplot.cpp,v 1.5 2008/06/26 09:42:56 panos Exp $
 *
 */

#include <iostream>
#include <string>
#include <vector>

#include <assert.h>

#include <stdlib.h>
#include <math.h>

#include "tokenize.h"

using namespace std;

int const QUANTILES = 1000;

int
main()
{
	string line;
	vector<long> expecteds;
	vector<long> actuals;

	/*
	 * Get the expected and actual number of references added
	 * to each article each month.
	 */
	cerr << "reading input..." << endl;
	while (getline(cin, line))
	{
		vector<string> fields;
		tokenize(line, fields, ":");
		assert(fields.size() == 7);

		long expected = atoi(fields[5].c_str());
		long actual = atoi(fields[6].c_str());
	       	if (expected > 0 && actual > 0) {
			expecteds.push_back(expected);
			actuals.push_back(actual);
		}
	}

	cerr << "finished input..." << endl;
	cerr << actuals.size() << " actuals " << expecteds.size() << " expecteds" << endl;
	cerr << "sorting..." << endl;

	sort(expecteds.begin(), expecteds.end());
	sort(actuals.begin(), actuals.end());
	
	cerr << "finished sorting..." << endl;
	
	cout << "#quantile\tpoint\t\texpecteds\tactuals" << endl;
	long n = expecteds.size();
	int t =  n / QUANTILES;
	long indx;
	for (int i = 1; i <= QUANTILES; i++) {
		indx = i * t;
		cout << i << "\t\t" << indx << "\t\t" << expecteds[indx] << "\t\t" << actuals[indx] << endl;
	}
}
