/*
 * Copyright (c) 2007, 2008 Diomidis Spinellis.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Publications using results from this work or its derivatives must
 *    acknowledge the work's origin by citing the corresponding August 2008
 *    Communications of the ACM paper.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: scan.l,v 1.16 2008/06/26 06:30:12 dds Exp $
 *
 * Go through a complete XML dump of Wikipedia, measuring the evolution
 * of various attributes.
 */


/* In a <text> tag: scan for references and redirects */
%x TEXT

/* In a reference link: skip verb and invalid references */
%x REF

/* In a user or talk page: skip it */
%x SKIPPAGE

%option noyywrap

%%

"<title>"[^<:]*:[^<]*"</title>"		BEGIN(SKIPPAGE);
"<title>"[^<:]*"</title>"		title(yytext);
"<timestamp>"[^<]*"</timestamp>"	timestamp(yytext);
"<username>"[^<]*"</username>"		username(yytext);
"<comment>moved [["[^\]:]+"]] to [["[^\]:]+"]]" move(yytext);
"<comment>"(([Rr][Vv])|([Rr]"evert"))	revert();
"<comment>BOT - rv"			revert();
"<ip>"					ip();
"<page>"				page_begin();
"<text xml:space=\"preserve\">"		BEGIN(TEXT);
"</page>"				page_end();
"<restrictions>"			restrictions();

<TEXT>"[["/[^#]				BEGIN(REF);
<TEXT>"#REDIRECT [["			{
						redirect();
						BEGIN(REF);
					}
<TEXT>"stub}}"				stub();
<TEXT>"</text>"				BEGIN(INITIAL);

<REF>[^[<:|\]#]{1,256}/[#|\]]		{
						reference(yytext);
						BEGIN(TEXT);
					}
<REF>.|\n				{
						unput(*yytext);
						BEGIN(TEXT);
					}

<SKIPPAGE>"</page>"			BEGIN(INITIAL);

<INITIAL,TEXT,SKIPPAGE>.|\n
